package be.ac.ulb.code.wit.resources.serializers;

import java.io.Reader;

import org.apache.abdera.model.Entry;
import org.apache.abdera.parser.ParseException;

import be.ac.ulb.code.wit.contacts.Contacts;
import be.ac.ulb.code.wit.contacts.Contacts.IdentifiedContact;

/**
 * An atom deserializer that create, update or delete 
 * contacts, using data from atom entries.
 * 
 * @author fpicalausa
 *
 */
public class ContactDeserializer {
	private final Contacts contacts;
	private IdentifiedContact identifiedContact;
	private boolean hasContact;
	private String group = null;
	
	/**
	 * Set the contact referred to by this deserializer. 
	 * @throws IllegalStateException if a contact had already been set.
	 */
	private void setContact(IdentifiedContact identifiedContact) throws IllegalStateException {
		if (hasContact) {
			throw new IllegalStateException();
		}
		
		this.identifiedContact = identifiedContact;
		this.hasContact = true;
	}

	/**
	 * Constructs an atom deserializer that update an existing contact 
	 * with data from atom entries.
	 * @param contacts The contacts list to be updated
	 * @param existingContact An existing contact to be updated.
	 * @param group The group of contacts from which the atom document is deserialized.
	 * @throws NotFoundException if the specified group is not valid
	 */
	public ContactDeserializer(final Contacts contacts, 
			final IdentifiedContact existingContact,
			final String group) throws NotFoundException {
		super();
		
		ContactSerializer.checkGroup(group);
		
		this.contacts = contacts;
		this.group = group;

		if (existingContact != null) {
			setContact(existingContact);
		}
	}

	/**
	 * Constructs an atom deserializer that creates new contacts
	 * with data from atom entries.
	 * @param contacts The contacts list to be updated
	 * @param existingContact An existing contact to be updated.
	 * @param group The group of contacts from which the atom document is deserialized.
	 * @throws NotFoundException if the specified group is not valid
	 */
	public ContactDeserializer(final Contacts contacts, final String group) {
		this(contacts, (IdentifiedContact)null, group);
	}

	/**
	 * Returns the contact under update, or the newly create contact if
	 * no contact had been specified when constructing the ContactDeserializer.
	 * @return the contact under update, or the newly create contact.
	 * @throws IllegalStateException if no contact has been provided to the
	 * 				deserializer, and no contact has been created from
	 * 				an atom entry after.
	 */
	public IdentifiedContact getIdentifiedContact() throws IllegalStateException {
		if (hasContact) {
			return identifiedContact;
		}
		throw new IllegalStateException("No contact has been created!");
	}

	/**
	 * Add our current contact to the group
	 */
	public void setContactGroup() {
		contacts.addContactToGroup(identifiedContact, group);
	}

	/**
	 * Remove our current contact from the group
	 */
	public void removeContact() {
		contacts.removeFromGroup(identifiedContact, group);
	}
    
    /**
     * Update the current contact with data from the given atom entry.
     * 
     * @param entry The atom entry to be parsed
     */
	public void updateContact(Entry entry) {
		//TODO: Fetch the data to create/update our contact from 
        //the parsed information
		
	}

    /**
     * Update the current contact with data from the given atom entry.
     * 
     * @param entry The atom entry to be parsed
     */
	public void updateContact(Reader data) {
		try {
			updateContact(AtomHelpers.parseEntry(data));
		}
		catch (ParseException e) {
			throw new MalformedRequestException(e);
		}
	}
}
