package be.ac.ulb.code.wit.resources.serializers;

import org.apache.abdera.Abdera;
import org.apache.abdera.factory.Factory;
import org.apache.abdera.model.Entry;
import org.apache.abdera.model.Feed;

import be.ac.ulb.code.wit.contacts.Contact;
import be.ac.ulb.code.wit.contacts.Contacts;
import be.ac.ulb.code.wit.contacts.Contacts.IdentifiedContact;

public class ContactSerializer {
	private final Contacts contacts;
	private final static Factory ABDERA_FACTORY = Abdera.getInstance().getFactory();
	private String group = null;

	/**
	 * Retrieves a contact in a contacts list by its identifier.
	 * @param contactId The identifier of the contact to be retrieved. 
	 * @param contacts The contacts list in which the specified contact is to be found.
	 * @return The contact as it appears in the contacts list.
	 * @throws NotFoundException The specified contact could not be found in the contacts list.
	 */
	private static IdentifiedContact getContact(final int contactId, Contacts contacts) throws NotFoundException {
		IdentifiedContact result = contacts.getContact(contactId);
		if (result == null) {
			throw new NotFoundException("The specified id could not be matched to a contact.");
		}
		return result;
	}

	/**
	 * Retrieves a contact in a contacts list by its identifier.
	 * @param contactId The identifier of the contact to be retrieved. 
	 * @param contacts The contacts list in which the specified contact is to be found.
	 * @return The contact as it appears in the contacts list.
	 * @throws NotFoundException The specified contact could not be found in the contacts list.
	 */
	public static IdentifiedContact getContact(final String contactId, Contacts contacts) throws NotFoundException {
		try {
			final int cid = Integer.parseInt(contactId);
			return getContact(cid, contacts);
		}
		catch (NumberFormatException e) {
			throw new NotFoundException("The specified id has an incorrect format.", e);
		}
	}
	
	/**
	 * Ensures that specified contact belongs to the group used by this serializer.
	 * @param contact The contact to be checked
	 * @return The contact passed as an argument if it belongs to the group of the serializer. Throws a NotFoundException otherwise.
	 * @throws NotFoundException The specified contact does not belong to the group.
	 */
	private IdentifiedContact checkContactInGroup(IdentifiedContact contact) throws NotFoundException {
		if (group != Contacts.GROUP_ALLCONTACTS) {
			if (!contacts.getContactGroups(contact).contains(group)) {
				throw new NotFoundException("The specified contact does no belong to the requested group");
			}
		}
		return contact;
	}
	
	/**
	 * Retrieves a contact in a contacts list by its identifier.
	 * @param contactId The identifier of the contact to be retrieved. 
	 * @return The contact as it appears in the contacts list.
	 * @throws NotFoundException The specified contact could not be found in the contacts list.
	 */
	public IdentifiedContact getContact(final String contactId) throws NotFoundException {
		return checkContactInGroup(getContact(contactId, contacts));
	}

	/**
	 * Construct a new ContactSerializer, using information
	 * from the given Contacts list and working in the context
	 * of the given group.
	 * 
	 * @param contacts The contacts list to be used.
	 * @param group The contacts group in which the serialization occurs.
	 * @throws NotFoundException if the specified group is not valid.
	 */
	public ContactSerializer(final Contacts contacts, final String group) throws NotFoundException {
		super();
		this.contacts = contacts;
		checkGroup(group);
		
		this.group = group;
	}

	/**
	 * Check that a given group has a valid name, or is the default group
	 * with all contacts. Otherwise, throw a NotFoundException.
	 * @param group The group to be checked.
	 * @throws NotFoundException if the specified group is not valid.
	 */
	public static void checkGroup(String group) {
		if (group != Contacts.GROUP_ALLCONTACTS) {
			if (!Contact.validGroupName(group)) {
				throw new NotFoundException();
			}
		}
	}

	/**
	 * Creates an atom entry representing a contact.
	 * @param identifiedContact The contact to be serialized as an atom entry.
	 * @param summary Whether only a summary of the information (to include into
	 * 				a feed) is to be generated.
	 * @return an atom entry representing the given contact.
	 */
	private Entry makeEntry(final IdentifiedContact identifiedContact, final boolean summary) {
		final Entry entry = ABDERA_FACTORY.newEntry();
		
		//Fill in the data

		return entry;
	}
	
	/**
	 * Make a String representation of a contact, as an atom entry.
	 * @param contact The contact to be represented as an atom entry.
	 * @return a String representation of the given contact, as an atom entry.
	 */
	public String makeStringEntry(IdentifiedContact contact) {
		return AtomHelpers.stringify(makeEntry(contact, false));
	}

	
	/**
	 * Generates an atom feed containing every contacts in the context group.
	 * @return an atom feed containing every contacts in the context group.
	 */
	public String makeFeed() {
		final Feed contactsFeed = Abdera.getInstance().getFactory().newFeed();
		
		//TODO: build the corresponding feed
		
		return AtomHelpers.stringify(contactsFeed);
	}
}
